CREATE OR REPLACE FUNCTION xml.f_patent_document(integer, text) RETURNS xml
    LANGUAGE plpgsql
    AS $_$
declare
  v_pub_id alias for $1;
  v_useropt alias for $2;
begin
  -- select xml.f_patent_document( xml.f_ucid2id('CN-102837504-A'), '');
  -- attachments-external
  -- select xml.f_patent_document( 127422754, '');

  return (
    select xmlelement(name "patent-document",
               xmlattributes(modified_load_id as "file-reference-id",
                             v_pub_id as "mxw-id",
                             ucid as ucid,
                             lang as lang,
                             country as country,
                             doc_number as "doc-number",
                             kind as kind,
                             to_char(published, 'YYYYMMDD') as "date",
                             to_char(produced, 'YYYYMMDD') as "date-produced",
                             family_id as "family-id",
                             status as "status",
                             case when withdraw=true then 'yes' end as "withdrawn"),
             xmlelement(name "bibliographic-data",
              (select xmlagg(content) from xml.t_publication_reference where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_us_sir_flag where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_application_reference where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_priority_claims where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_dates_of_public_availability where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_rule_47_flag where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_term_of_grant where publication_id=v_pub_id ),
              (select xmlelement(name "technical-data",
                (select xmlagg(content) from xml.t_classification_ipc where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_classification_locarno where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_classifications_ipcr where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_classification_national where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_classification_ecla where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_classifications_cpc where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_f_term_info where publication_id=v_pub_id),
                -- sub function for just titles
                (select xml.f_invention_title(v_pub_id, v_useropt)),
                (select xmlagg(content) from xml.t_citations where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_figures where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_field_of_search where publication_id=v_pub_id),
                (select xmlagg(content) from xml.t_statistical_information where publication_id=v_pub_id)
              )),
              (select xmlagg(content) from xml.t_related_documents where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_parties where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_international_convention_data where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_designated_states where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_office_specific_data where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_ifi_integrated_content where publication_id=v_pub_id ),
              (select xmlagg(content) from xml.t_ifi_annotated_data where publication_id=v_pub_id )
            ) , -- end bibliographic-data
            (select xml.f_abstract(v_pub_id, v_useropt)),
            (select xml.f_description(v_pub_id, v_useropt)),
            (select xml.f_claims(v_pub_id, v_useropt)),
            (select xmlagg(content) from xml.t_amended_claims where publication_id=v_pub_id ),
            (select xmlagg(content) from xml.t_drawings where publication_id=v_pub_id ),
            -- ALPA-2386
            (select xmlagg(content) from xml.t_attachments_external where publication_id=v_pub_id ),
            (select xmlagg(content) from xml.t_search_report_data where publication_id=v_pub_id ),
            (select xmlagg(content) from xml.t_keywords where publication_id=v_pub_id ),
            (select xmlagg(content) from xml.t_legal_status where publication_id=v_pub_id ),
            (select xmlagg(content) from xml.t_revision_history where publication_id=v_pub_id ),
            (select xml.f_copyright( v_pub_id, v_useropt ) ) )
    from xml.t_patent_document_values
    where publication_id=v_pub_id
  );
end;
$_$;

