CREATE OR REPLACE FUNCTION xml.f_patent_document_s(integer, text) RETURNS SETOF xml
    LANGUAGE sql
    AS $_$
  select xmlelement(name "patent-document",
             xmlattributes(modified_load_id as "file-reference-id",
                           $1 as "mxw-id",
                           ucid as ucid,
                           lang as lang,
                           country as country,
                           doc_number as "doc-number",
                           kind as kind,
                           to_char(published, 'YYYYMMDD') as "date",
                           to_char(produced, 'YYYYMMDD') as "date-produced",
                           family_id as "family-id",
                           status as "status",
                           case when withdraw=true then 'yes' end as "withdrawn"),
           xmlelement(name "bibliographic-data",
            (select xmlagg(content) from xml.t_publication_reference where publication_id=$1 ),
            (select xmlagg(content) from xml.t_us_sir_flag where publication_id=$1 ),
            (select xmlagg(content) from xml.t_application_reference where publication_id=$1 ),
            (select xmlagg(content) from xml.t_priority_claims where publication_id=$1 ),
            (select xmlagg(content) from xml.t_dates_of_public_availability where publication_id=$1 ),
            (select xmlagg(content) from xml.t_rule_47_flag where publication_id=$1 ),
            (select xmlagg(content) from xml.t_term_of_grant where publication_id=$1 ),
            (select xmlelement(name "technical-data",
              (select xmlagg(content) from xml.t_classification_ipc where publication_id=$1),
              (select xmlagg(content) from xml.t_classification_locarno where publication_id=$1),
              (select xmlagg(content) from xml.t_classifications_ipcr where publication_id=$1),
              (select xmlagg(content) from xml.t_classification_national where publication_id=$1),
              (select xmlagg(content) from xml.t_classification_ecla where publication_id=$1),
              (select xmlagg(content) from xml.t_classifications_cpc where publication_id=$1),
              (select xmlagg(content) from xml.t_f_term_info where publication_id=$1),
              (select xml.f_invention_title($1, $2)),
              (select xmlagg(content) from xml.t_citations where publication_id=$1),
              (select xmlagg(content) from xml.t_figures where publication_id=$1),
              (select xmlagg(content) from xml.t_field_of_search where publication_id=$1),
              (select xmlagg(content) from xml.t_statistical_information where publication_id=$1)
            )),
           (select xmlagg(content) from xml.t_related_documents where publication_id=$1 ),
           (select xmlagg(content) from xml.t_parties where publication_id=$1 ),
           (select xmlagg(content) from xml.t_international_convention_data where publication_id=$1 ),
           (select xmlagg(content) from xml.t_designated_states where publication_id=$1 ),
           (select xmlagg(content) from xml.t_office_specific_data where publication_id=$1 ),
           (select xmlagg(content) from xml.t_ifi_integrated_content where publication_id=$1 ),
           (select xmlagg(content) from xml.t_ifi_annotated_data where publication_id=$1 )
         ) ,  -- end bibliographic-data
         (select xml.f_abstract($1, $2)),
         (select xml.f_description($1, $2)),
         (select xml.f_claims($1, $2)),
         (select xmlagg(content) from xml.t_amended_claims where publication_id=$1 ),
         (select xmlagg(content) from xml.t_drawings where publication_id=$1 ),
          -- ALPA-2386
         (select xmlagg(content) from xml.t_attachments_external where publication_id=$1 ),
         (select xmlagg(content) from xml.t_search_report_data where publication_id=$1 ),
         (select xmlagg(content) from xml.t_keywords where publication_id=$1 ),
         (select xmlagg(content) from xml.t_legal_status where publication_id=$1 ),
         (select xmlagg(content) from xml.t_revision_history where publication_id=$1 ),
         (select xml.f_copyright( $1, $2 ) ) )
  from xml.t_patent_document_values
  where publication_id=$1
$_$;

