TRUNCATE TABLE xml.t_sys_data;


CREATE OR REPLACE FUNCTION xml.f_alter_copyright () RETURNS boolean
    LANGUAGE 'plpgsql'
AS $BODY$

declare
  v_bool boolean;
begin
  -- alter table xml.t_sys_data drop column copyright_key;
  -- select xml.f_alter_copyright();
  select exists into v_bool (
    select 1
    from information_schema.columns 
    where table_schema='xml' AND table_name='t_sys_data' AND column_name='copyright_key'
  );

  if( v_bool != true ) then
    alter table xml.t_sys_data add column copyright_key text;
  end if;
  return v_bool;
end;
$BODY$;

SELECT xml.f_alter_copyright();

DROP FUNCTION xml.f_alter_copyright();

INSERT INTO xml.t_sys_data ( copyright_key, copyright ) VALUES (
  'ifi',
  'User acknowledges that Fairview Research and its third party providers retain all right, title and interest in and to this xml under applicable copyright laws. User acquires no ownership rights to this xml including but not limited to its format. User hereby accepts the terms and conditions of the License Agreement.'
);

INSERT INTO xml.t_sys_data ( copyright_key, copyright ) VALUES (
  'epo',
  'This record contains data sourced from EPO databases (C)European Patent Organization.'
);

INSERT INTO xml.t_sys_data ( copyright_key, copyright ) VALUES (
  'cipo',
  'This record contains data licensed by Her Majesty the Queen in right of Canada, as represented by the Minister of Industry, the minister responsible for the administration of the Canadian Intellectual Property Office.'
);


CREATE OR REPLACE FUNCTION xml.f_copyright( integer, text) RETURNS xml
    LANGUAGE 'plpgsql'
AS $BODY$
declare
  v_pub_id alias for $1;
  v_useropt alias for $2;
  v_xml xml;
  v_ctry text;
  v_ids text[];
  
begin
-- select * from xml.t_sys_data;
-- US: select xml.f_copyright( 124000010, null );
-- EP: select xml.f_copyright( 124019709, null );

  select country into v_ctry from xml.t_patent_document_values where publication_id = v_pub_id;
  -- always include ifi
  select array_append( v_ids, 'ifi') into v_ids;
  -- CA data gets CIPO attribution
  if( v_ctry = 'CA' ) then
    select array_append( v_ids, 'cipo') into v_ids;
  end if;
  -- EP and any country with DOCDB data gets EPO attribution
  if( v_ctry = 'EP' || xml.f_has_docdb( v_pub_id ) ) then
    select array_append( v_ids, 'epo') into v_ids;
  end if;
  
  
  select xmlagg( xmlelement( name "copyright",
            xmlattributes( copyright_key as "copyright-id"),
            copyright ) )
    into v_xml from xml.t_sys_data where copyright_key = any ( v_ids );
  return v_xml;
end;
$BODY$;

