DROP SCHEMA reports CASCADE;
CREATE SCHEMA reports;

--
-- Name: f_csv_abstract_en(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_abstract_en(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_rec record;
        v_cnt integer;
        v_lang text;
        v_ord integer;
        v_content xml;
        v_node xml;
        v_txt text;

/*
  select t1.publication_id,ucid
  from xml.t_patent_document_values t1
    inner join xml.t_abstract t2
      on (t1.publication_id = t2.publication_id)
  limit 10;

  select * from xml.t_abstract where publication_id=119008180;
*/

-- select reports.f_csv_abstract_en( 119008180 ); -- xml.f_ucid2id('US-5547684-A') );
-- select reports.f_csv_abstract_en( xml.f_ucid2id('EP-0700001-A') );
-- select reports.f_csv_abstract_en( 119010254 );
-- select reports.f_csv_abstract_en( xml.f_ucid2id('US-8056298-B2') );

begin

  create temp table ab_tbl ( pub_lang varchar(2), ab_lang varchar(2), x xml) on commit drop;

  for v_rec in
        select t1.lang as pub_lang, t2.*
        from xml.t_patent_document_values t1
          inner join xml.t_abstract t2
            on (t1.publication_id=t2.publication_id)
        where t1.publication_id=$1 loop
    insert into ab_tbl (pub_lang, ab_lang, x)
      values ( v_rec.pub_lang,
               xml.f_getAttribute('lang', v_rec.content),
               v_rec.content
      );
  end loop;

  select ab_lang,
    case when ab_lang = 'EN'     then 0
         when ab_lang = pub_lang then 1
         when ab_lang = 'DE'     then 2
         when ab_lang = 'FR'     then 3
         else 4
    end as ord, x
    into v_lang, v_ord, v_content
  from ab_tbl
  order by ord limit 1;

  select count(*) into v_cnt from ab_tbl;

  if (v_cnt > 0) then
    if (xml.f_nodeExists('//abstract[@lang="'||v_lang||'"]', v_content) ) then
      select xml.f_node2string( xml.f_findNode('//abstract[@lang="'||v_lang||'"]', v_content)::text, 0 ) into v_txt;
    end if;
  end if;
  
  if (v_txt is not null) then
    return v_txt;
  end if;
  
  return ( reports.f_csv_util_msg_notavail()  );
end;

$_$;


--
-- Name: f_csv_agent(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_agent(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$


declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_has_epo boolean := false;
        v_content xml;

-- select reports.f_csv_agent(xml.f_ucid2id('CN-101642422-B'));

begin

  select content into v_content from xml.t_parties where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;
  
  foreach v_node in array
    ( select xml.f_findNodes('//agent[@format="epo"]', v_content) ) loop

    select reports.f_csv_name(v_node) into v_name;
    
    if v_name is not null then
      v_has_epo := true;
      select array_append(v_names, v_name) into v_names;
    end if;
    
  end loop;

  -- if EPO format doesn't exist, settle for original
  if (v_has_epo = false) then
    foreach v_node in array
      ( select xml.f_findNodes('//agent[@format="original"]', v_content) ) loop
      
      select reports.f_csv_name(v_node) into v_name;
    
      if v_name is not null then
        select array_append(v_names, v_name) into v_names;
      end if;
    end loop;
  end if;
  
  if array_length(v_names, 1) > 0 then
    return array_to_string(v_names, ' | ');
  end if;
  
  return reports.f_csv_util_msg_notavail();
  
end;
$_$;


--
-- Name: f_csv_application_country(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_country(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_ucid text;
        v_a text[];

begin
  -- select reports.f_csv_application_country( xml.f_ucid2id('US-5551212-A') );
  select content into v_content from xml.t_application_reference where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;

  select xml.f_getAttribute('ucid', v_content) into v_ucid;
  if( v_ucid is not null ) then
    select regexp_split_to_array(v_ucid, '-') into v_a;
    return v_a[1];
  end if;
  
  return ( reports.f_csv_util_msg_notavail() );
end;
$_$;


--
-- Name: f_csv_application_date(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_date(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_txt text;

-- select reports.f_csv_application_date( xml.f_ucid2id('EP-0700000-A2') );
begin
  select xml.f_textContent('//application-reference/document-id/date', content) into v_txt
  from xml.t_application_reference
  where publication_id=$1
  limit 1;

  if (v_txt is null) then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  
  return to_char(v_txt::date, 'YYYY-MM-DD')::text;
end;
$_$;


--
-- Name: f_csv_application_kind(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_kind(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_ucid text;
        v_a text[];

begin
  -- select reports.f_csv_application_kind( xml.f_ucid2id('US-5551212-A') );
  select content into v_content from xml.t_application_reference where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;

  select xml.f_getAttribute('ucid', v_content) into v_ucid;
  if( v_ucid is not null ) then
    select regexp_split_to_array(v_ucid, '-') into v_a;
    return v_a[3];
  end if;
  
  return ( reports.f_csv_util_msg_notavail() );
end;
$_$;


--
-- Name: f_csv_application_number(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_number(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_ucid text;
        v_a text[];

begin
  -- select reports.f_csv_application_number( xml.f_ucid2id('US-5551212-A') );
  select content into v_content from xml.t_application_reference where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;

  select xml.f_getAttribute('ucid', v_content) into v_ucid;
  if( v_ucid is not null ) then
    select regexp_split_to_array(v_ucid, '-') into v_a;
    return v_a[2];
  end if;
  
  return ( reports.f_csv_util_msg_notavail() );
end;
$_$;


--
-- Name: f_csv_application_ucid(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_ucid(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_ucid text;

begin
  -- select reports.f_csv_application_number( xml.f_ucid2id('EP-0700000-B1') );
  select content into v_content from xml.t_application_reference where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;

  select xml.f_getAttribute('ucid', v_content) into v_ucid;
  if( v_ucid is not null ) then
    return v_ucid;
  end if;
  return ( reports.f_csv_util_msg_notavail() );
end;
$_$;


--
-- Name: f_csv_application_year(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_application_year(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_txt text;

-- select reports.f_csv_application_year( xml.f_ucid2id('EP-0700000-A2') );
begin
  select xml.f_textContent('//application-reference/document-id/date', content) into v_txt
  from xml.t_application_reference
  where publication_id=$1
  limit 1;

  if (v_txt is null) then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  
  return to_char(v_txt::date, 'YYYY')::text;
end;
$_$;


--
-- Name: f_csv_assignee(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_assignee(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;
        v_elements text[];
        v_element text;
        v_has_original boolean := false;

-- select reports.f_csv_assignee(xml.f_ucid2id('US-5551212-A'));

begin

  select content into v_content from xml.t_parties where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  select array_append(v_elements, 'applicant') into v_elements;
  select array_append(v_elements, 'assignee') into v_elements;
  
  create temp table t_n_asg (format text, nm text);

  foreach v_element in array v_elements loop
    foreach v_node in array
      ( select xml.f_findNodes('//' || v_element || '[@format="original"]', v_content) ) loop

      select reports.f_csv_name(v_node) into v_name;
      if v_name is not null then
        v_has_original := true;
        insert into t_n_asg (nm) values (v_name);
      end if;
    
    end loop;
  end loop;

  if (v_has_original = false) then
    foreach v_element in array v_elements loop
      foreach v_node in array
        ( select xml.f_findNodes('//' || v_element || '[@format="epo"]', v_content) ) loop

        select reports.f_csv_name(v_node) into v_name;
        if v_name is not null then
          insert into t_n_asg (nm) values (v_name);
        end if;
        
      end loop;
    end loop;
  end if;
  
  for v_name in select distinct nm from t_n_asg loop
    select array_append(v_names, v_name) into v_names;
  end loop;

  drop table t_n_asg;
  
  if array_length(v_names, 1) > 0 then
    return array_to_string(v_names, ' | ');
  end if;
  
  return reports.f_csv_util_msg_notavail();
end;

$_$;


--
-- Name: f_csv_assignee_std(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_assignee_std(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;
        v_elements text[];
        v_element text;

-- select reports.f_csv_assignee_std(xml.f_ucid2id('US-5551212-A'));

begin

  select content into v_content from xml.t_parties where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  select array_append(v_elements, 'applicant') into v_elements;
  select array_append(v_elements, 'assignee') into v_elements;

  foreach v_element in array v_elements loop
    foreach v_node in array
      ( select xml.f_findNodes('//' || v_element || '[@format="epo"]', v_content) ) loop

      select reports.f_csv_name(v_node) into v_name;
      if v_name is not null then
        select array_append(v_names, v_name) into v_names;
      end if;
    
    end loop;
  end loop;
 
  if array_length(v_names, 1) > 0 then
    return array_to_string(v_names, ' | ');
  end if;
  
  return reports.f_csv_util_msg_notavail();
end;

$_$;


--
-- Name: f_csv_claims_en(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_claims_en(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_node xml;
        v_array text[] = '{}';

my_output text [];
    my_tmp text;

  -- select * from xml.t_claims where publication_id=xml.f_ucid2id('US-5526727-A');
  -- select reports.f_csv_claims_en(xml.f_ucid2id('US-5526727-A'));
  
begin

  for v_content in select content from xml.t_claims where publication_id=$1 loop
    foreach v_node in array
      ( select xml.f_findNodes('//claims[@lang="EN"]/claim', v_content) ) loop

      v_array = array_append( v_array, xml.f_node2string( v_node::text, 0 ) );
      
    end loop; 
  end loop;

  if(v_array is null or array_length(v_array, 1) is null or array_length(v_array, 1) < 1) then
    return(reports.f_csv_util_msg_notavail());
  end if;
  
  return( array_to_string(v_array, ' | '));
      
end;
$_$;


--
-- Name: f_csv_claims_first(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_claims_first(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_rec record;
        v_cnt integer;
        v_lang text;
        v_ord integer;
        v_content xml;
        v_node xml;
        v_claim text;

-- WO-2003104270-A3
-- WO-1983001816-A1
-- WO-2003104808-A1
-- WO-2003104441-A3
-- WO-1982000069-A1
-- WO-2003104031-A3
-- select length( reports.f_csv_claims_first( xml.f_ucid2id('EP-0700000-B1') ) );
-- select length( reports.f_csv_claims_en( xml.f_ucid2id('EP-0700000-B1') ) );
-- select reports.f_csv_claims_first( 71352670 );

begin
  create temp table clm_tbl_first ( pub_lang varchar(2), clm_lang varchar(2), x xml) on commit drop;

  for v_rec in
        select t1.lang as pub_lang, t2.*
        from xml.t_patent_document_values t1
          inner join xml.t_claims t2
            on (t1.publication_id=t2.publication_id)
        where t1.publication_id=$1 loop
    insert into clm_tbl_first (pub_lang, clm_lang, x)
      values ( v_rec.pub_lang,
               xml.f_getAttribute('lang', v_rec.content),
               v_rec.content
      );
  end loop;

  select clm_lang,
    case when clm_lang = 'EN' then 0
         when clm_lang = pub_lang then 1
         else 2
    end as ord, x
    into v_lang, v_ord, v_content
  from clm_tbl_first
  order by ord limit 1;

  select count(*) into v_cnt from clm_tbl_first;

  -- raise notice 'clm_tbl_first: %,%,%', v_lang,v_ord,v_content;

  if (v_cnt > 0) then
    if ( xml.f_nodeExists('//claims[@lang="'||v_lang||'"]', v_content) ) then
      foreach v_node in array
        ( select xml.f_findNodes('//claims[@lang="'||v_lang||'"]/claim', v_content) ) loop
        select xml.f_node2string( v_node::text, 0 ) into v_claim;
        exit; -- we are done
      end loop;
    end if;
  end if;

  if( v_claim is null ) then
    return(reports.f_csv_util_msg_notavail());
  end if;

  return( v_claim );

end;
$_$;


--
-- Name: f_csv_claims_original(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_claims_original(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_rec record;
        v_cnt integer;
        v_lang text;
        v_ord integer;
        v_content xml;
        v_node xml;
        v_array text[];

-- ALPA-1975
-- WO-2003104270-A3
-- WO-1983001816-A1
-- WO-2003104808-A1
-- WO-2003104441-A3
-- WO-1982000069-A1
-- WO-2003104031-A3
-- select reports.f_csv_claims_original( xml.f_ucid2id('WO-2003104270-A3') );
-- select reports.f_csv_claims_original( 71352670 );

begin
  create temp table clm_tbl ( pub_lang varchar(2), clm_lang varchar(2), x xml) on commit drop;

  for v_rec in
        select t1.lang as pub_lang, t2.*
        from xml.t_patent_document_values t1
          inner join xml.t_claims t2
            on (t1.publication_id=t2.publication_id)
        where t1.publication_id=$1 loop
    insert into clm_tbl (pub_lang, clm_lang, x)
      values ( v_rec.pub_lang,
               xml.f_getAttribute('lang', v_rec.content),
               v_rec.content
      );
  end loop;

  select clm_lang,
    case when clm_lang = pub_lang then 0
         else 1
    end as ord, x
    into v_lang, v_ord, v_content
  from clm_tbl
  where clm_lang != 'EN'
  order by ord limit 1;

  select count(*) into v_cnt from clm_tbl;

  -- raise notice 'clm_tbl: %,%,%', v_lang,v_ord,v_content;

  if (v_cnt > 0) then
    if ( xml.f_nodeExists('//claims[@lang="'||v_lang||'"]', v_content) ) then
      foreach v_node in array
        ( select xml.f_findNodes('//claims[@lang="'||v_lang||'"]/claim', v_content) ) loop
        
        v_array = array_append( v_array, xml.f_node2string( v_node::text, 0 ) );
      
      end loop;
    end if;
  end if;

  if(v_array is null or array_length(v_array, 1) is null or array_length(v_array, 1) < 1) then
    return(reports.f_csv_util_msg_notavail());
  end if;

  return( array_to_string(v_array, ' | '));

end;
$_$;


--
-- Name: f_csv_correspondent(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_correspondent(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- 20121209-- jmb --first cut


declare my_rec record;
	my_output text;
	my_tmp text;

begin

    select unnest(xpath('//correspondence-address/addressbook/last-name/text() ', content))::text into my_output from xml.t_parties
       where publication_id=$1;

    if found
    then
        return (my_output);
    else
     	return(reports.f_csv_util_msg_notavail());
 
    end if;
end;
$_$;


--
-- Name: f_csv_cpc_all(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_cpc_all(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare my_content xml;
my_array text [];
my_output text[];
my_tmp text;

-- 20130131 jmb -- first cut
-- cpc function !!!!


begin
    -- read content from db into my_content
    select content into my_content
    from xml.t_classifications_cpc
    where publication_id=$1;
    
    -- check to see if I got something
    if found then    
          
        select xpath('//classification-cpc/text()', my_content)::text into my_array;
        
        if (my_array is null) then
          return ( reports.f_csv_util_msg_notavail() );
        end if;
        
        foreach my_tmp in array my_array
    loop
       my_output := array_append ( my_output, regexp_replace(rtrim(regexp_replace(rtrim(my_tmp),'\d{8}.*$', '' )),' +',' ','g'));
        end loop;        
  
    end if;

    -- check for null output
    if my_output is null
    then
        return(reports.f_csv_util_msg_notavail());
    else
        return ( array_to_string(my_output, ' | ' ));
    end if;
end;
$_$;


--
-- Name: f_csv_cpc_fi(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_cpc_fi(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_fi text;
        v_node xml;        

begin

    select content into v_content from xml.t_classifications_cpc where publication_id=$1;
    if not found then
      return reports.f_csv_util_msg_notavail();
    end if;

    -- select * from xml.t_classifications_cpc limit 1;
    -- select reports.f_csv_cpc_fi( xml.f_ucid2id('EP-0700000-A2') );
    
    foreach v_node in array
      ( select xml.f_findNodes('./classification-cpc', v_content) ) loop

      if( position('FI' in ( regexp_split_to_array( xml.f_textContent( '.', v_node ),  E'\\s+' ))[4] ) = 1 ) then
        select xml.f_textContent( '.', v_node) into v_fi;
      end if;
      
    end loop;
    
    -- check for null output
    if v_fi is null
    then
        return reports.f_csv_util_msg_notavail();
    else
        return v_fi;
    end if;
end;
$_$;


--
-- Name: f_csv_date_produced(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_date_produced(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_produced date;

begin

-- select reports.f_csv_date_produced( xml.f_ucid2id('EP-0700000-B1') );

    select produced into v_produced
    from xml.t_patent_document_values
    where publication_id=$1;

  return to_char(v_produced::date, 'yyyy-mm-dd')::text;
    
end;
$_$;


--
-- Name: f_csv_designated_states(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_designated_states(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$


declare v_ctry text;
        v_cnt integer;
        v_array text[];
         my_output text[];
	my_tmp text;

-- select reports.f_csv_designated_states(xml.f_ucid2id('WO-2003104808-A1'));
begin

  create temp table ds_tbl ( ctry varchar(2) ) on commit drop;

  for v_ctry in
      select unnest(xpath('//country/text()', content))::text
      from xml.t_international_convention_data
      where publication_id=$1 loop

    insert into ds_tbl (ctry) values (v_ctry);
    
  end loop;

  select count(*) into v_cnt from ds_tbl;
  if (v_cnt > 0) then
    for v_ctry in select distinct ctry from ds_tbl order by ctry asc loop
      v_array = array_append(v_array, v_ctry);
    end loop;
  end if;

  if (v_cnt < 1) then
    return(reports.f_csv_util_msg_notavail());
  end if;
  
  return( array_to_string(v_array, ' | '));
     
end;
$_$;


--
-- Name: f_csv_ecla(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ecla(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121203 -- jmb -- added ipc_padding function
-- hacked 20121206 -- jmb -- wrapped foreach with null check 
    -- commented out ipc_padding fuction ... client changed their mind
    -- added "not available" message
-- 20121212 --jmb removed extra spaces and fixed |

declare v_classes text[];
	ecla_raw text;
	ecla_out text[];
begin
  select xpath('//classification-ecla/classification-symbol/text()', content)::text
    into v_classes
  from xml.t_classification_ecla
  where publication_id=$1;
  
  if v_classes is not null
  then
   foreach ecla_raw in array(v_classes) loop
    -- convert to mpi ugly format
    -- ecla_raw = reports.f_csv_ipc_pad(ecla_raw);
    ecla_out = array_append(ecla_out, regexp_replace(ecla_raw,' +',' ','g')); 
   end loop;
  end if;

  if v_classes is not null
    then
        return array_to_string(ecla_out, ' | ');
    else
        return (reports.f_csv_util_msg_notavail());
    end if;
    
end;
$_$;


--
-- Name: f_csv_espacenet_link(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_espacenet_link(integer) RETURNS text
    LANGUAGE plperlu
    AS $_$

# select reports.f_csv_espacenet_link(xml.f_ucid2id('US-D566998-S1'));
# "http://worldwide.espacenet.com/publicationDetails/biblio?CC=US&NR=D566998&KC=S1&FT=D"
my $id = shift(@_);

### short cut exit
my $ret = spi_exec_query("select xml.f_has_docdb($id) as ret");
if($ret->{rows}[0]->{ret} eq 'f') {
  return "No Data";
}

my $plan = spi_prepare('select country, doc_number, kind from xml.t_patent_document_values where publication_id=$1', 'integer');
my $rv = spi_exec_prepared($plan, $id);
spi_freeplan($plan);
my $nrows = $rv->{processed};
if(!$nrows) { "No Data"; }

# always take first row
my ($ctry, $dnum, $kind, $ft) = (
  $rv->{rows}[0]->{country},
  $rv->{rows}[0]->{doc_number},
  $rv->{rows}[0]->{kind},
  ''
);

# some exceptions ...
if($ctry eq 'US' && length($dnum) >= 10) {
  # us applications
  my ($y, $n) = ( $dnum =~ /^(\d{4})(\d{7})$/ );
  $dnum = sprintf("%s%06d", $y, $n);
}
if( $ctry eq 'US' && $dnum =~ /^D/ ) {
  $dnum = "$dnum$kind";
  $ft = 'D';
}
if($ctry eq 'WO') {
  my ($y0, $y1, $n) = ( $dnum =~ /^(\d{2})(\d{2})(\d{6})$/ );
  my $year = "$y0$y1";
  if($year < 2002) {
    $dnum = sprintf("02d%05d", $y1, $n);
  }
  elsif($year == 2002 && $n >= 51231) {
    $dnum = sprintf("02d%06d", $y1, $n);
  }
  elsif($year == 2003 && $n <= 107732) {
    $dnum = sprintf("02d%06d", $y1, $n);
  }
}

#if( $kind =~ /^T/ ) {
#  return sprintf("http://worldwide.espacenet.com/publicationDetails/biblio?CC=%s&NR=%s%s&KC=%s", $ctry, $dnum, $kind, $kind);
#}
#else {
return sprintf("http://worldwide.espacenet.com/publicationDetails/biblio?CC=%s&NR=%s&KC=%s%s", $ctry, $dnum, $kind, ($ft) ? '&FT=D' : '');
#}

$_$;


--
-- Name: f_csv_examiner(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_examiner(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121206 --jmb


declare my_rec record;
	my_output text;
	my_tmp text;


begin

    -- grab <name> data if it exists
    select unnest(xpath('//examiners/examiner/addressbook/name/text() ', content))::text into my_output from xml.t_parties
       where publication_id=$1;

    -- otherwise dig for <last-name> <first-name.
    if my_output is null
    then
     for my_rec in
       select unnest(xpath('//examiners/examiner/addressbook/last-name/text() ', content))::text as lastname,
              unnest(xpath('//examiners/examiner/addressbook/first-name/text() ', content))::text as firstname
        from xml.t_parties
        where publication_id=$1 loop
        my_output = concat(my_rec.lastname, ', ', my_rec.firstname);
      end loop;
    end if;


 if my_output is null then
  	return(reports.f_csv_util_msg_notavail());
 else	
    return ( my_output );
 end if;
end;

$_$;


--
-- Name: f_csv_family_id(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_family_id(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_family_id int;

begin

-- select reports.f_csv_family_id( xml.f_ucid2id('EP-0700000-B1') );

    select family_id into v_family_id
    from xml.t_patent_document_values
    where publication_id=$1;

    if (v_family_id < 1) then
      return ( reports.f_csv_util_msg_notavail());
    end if;

    return v_family_id;
    
end;
$_$;


--
-- Name: f_csv_family_members(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_family_members(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121206 jmb
-- ...hard to test on smallDB

declare v_family_id int;
	v_ucid text;
	v_output text [];

begin

  -- select reports.f_csv_family_members( xml.f_ucid2id('EP-0700000-B1') );
  select family_id into v_family_id
  from xml.t_patent_document_values
  where publication_id=$1;

  if (v_family_id < 1) then
    return ( reports.f_csv_util_msg_notavail());
  end if;
    
    -- grab ucids for family into output array ... and not primary document
  for v_ucid in
    select ucid
    from xml.t_patent_document_values
    where family_id = v_family_id
      and publication_id != $1
    order by ucid loop
   
    select array_append( v_output, v_ucid ) into v_output;
    
  end loop;

  if    (v_output is null)
     or (array_length(v_output, 1) is null)
     or (array_length(v_output, 1) < 1)	then
    return(reports.f_csv_util_msg_notavail());
  end if;

  return (array_to_string(v_output, ' | '));

end;
$_$;


--
-- Name: f_csv_forward_citations(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_forward_citations(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- select reports.f_csv_forward_citations( xml.f_ucid2id('US-5551212-A') );
-- select ucid from api.f_forward_citations( xml.f_ucid2id( 'US-5551212-A') );
declare v_ucid text;
        v_ucids text[] := '{}';

begin

  select ucid into v_ucid from xml.t_patent_document_values where publication_id=$1;
  if not found then
    return (reports.f_csv_util_msg_notavail() );
  end if;

  for v_ucid in select ucid from api.f_forward_citations( $1 ) order by ucid loop
    select array_append(v_ucids, v_ucid) into v_ucids;
  end loop;
  
  if (array_length(v_ucids, 1) is null or array_length(v_ucids, 1) < 1 ) then
    return (reports.f_csv_util_msg_notavail() );
  end if;
  
  return array_to_string( v_ucids, ' | ' );

end;
$_$;


--
-- Name: f_csv_forward_citations_ex(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_forward_citations_ex(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- select reports.f_csv_forward_citations( xml.f_ucid2id('US-5551212-A') );
-- select ucid from api.f_forward_citations( xml.f_ucid2id( 'US-5551212-A') );
declare v_ucid text;
        v_ucids text[] := '{}';

begin

  select ucid into v_ucid from xml.t_patent_document_values where publication_id=$1;
  if not found then
    return (reports.f_csv_util_msg_notavail() );
  end if;

  for v_ucid in select ucid from api.f_forward_citations( $1 ) order by ucid loop
    select array_append(v_ucids, v_ucid) into v_ucids;
  end loop;

  if (array_length(v_ucids, 1) is null or array_length(v_ucids, 1) < 1 ) then
    return (reports.f_csv_util_msg_notavail() );
  end if;

  return array_to_string( v_ucids, ' | ' );

end;
$_$;


--
-- Name: f_csv_fterms_all(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_fterms_all(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- first cut -- 20121210 -- jmb

declare my_content xml;
	my_array text [];
	my_output text[];
	my_tmp text;

begin
    -- read content from db into my_content
    select content into my_content
    from xml.t_f_term_info
    where publication_id=$1;
    
    -- check to see if I got something
    if found
    then    
        
        select xpath('//f-term/text()', my_content)::text into my_array;
        foreach my_tmp in array my_array
	    loop
	       my_output := array_append ( my_output, my_tmp);
        end loop;        
  
    end if;

    -- check for null output
    if my_output is null
    then
        return(reports.f_csv_util_msg_notavail());
    else
        return ( array_to_string(my_output, ' | ' ));
    end if;
end;
$_$;


--
-- Name: f_csv_ifi_adjusted_expiration(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_adjusted_expiration(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_exp text;
        v_content xml;

/*
<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_status(160182314);
   xml.f_ucid2id('US-5551212-A'));
 select reports.f_csv_ifi_adjusted_expiration( xml.f_ucid2id('US-9165295-B2') );
*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array ( select xml.f_findNodes('//ifi-patent-status', v_content) ) loop
    select xml.f_getAttribute('adjusted-expiration', v_node) into v_exp;
    if v_exp is not null then
      return v_exp;
    end if;
  end loop;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_assignee(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_assignee(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;

/*

<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_assignee_current( xml.f_ucid2id('US-5551212-A'));

*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//ifi-standardized-name', v_content ) ) loop
    select reports.f_csv_name( v_node ) into v_name;
    if v_name is not null then
      select array_append( v_names, v_name ) into v_names;
    end if;
  end loop;

  if array_length( v_names, 1 ) > 0 then
    return array_to_string( v_names, '|' );
  end if;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_assignee_current(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_assignee_current(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;

/*

<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_assignee_current( xml.f_ucid2id('US-5551212-A'));

*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//ifi-standardized-name-current', v_content ) ) loop
    select reports.f_csv_name( v_node ) into v_name;
    if v_name is not null then
      select array_append( v_names, v_name ) into v_names;
    end if;
  end loop;

  if array_length( v_names, 1 ) > 0 then
    return array_to_string( v_names, '|' );
  end if;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_assignee_original(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_assignee_original(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;

/*

<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_assignee_original( xml.f_ucid2id('US-5551212-A'));

*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//ifi-standardized-name', v_content ) ) loop
    select reports.f_csv_name( v_node ) into v_name;
    if v_name is not null then
      select array_append( v_names, v_name ) into v_names;
    end if;
  end loop;

  if array_length( v_names, 1 ) > 0 then
    return array_to_string( v_names, '|' );
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//ifi-standardized-probable', v_content ) ) loop
    select reports.f_csv_name( v_node ) into v_name;
    if v_name is not null then
      select array_append( v_names, v_name ) into v_names;
    end if;
  end loop;

  if array_length( v_names, 1 ) > 0 then
    return array_to_string( v_names, '|' );
  end if;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_assignee_probable(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_assignee_probable(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_content xml;

/*

<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_assignee_probable(xml.f_ucid2id('US-20150304621-A1'));

*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//ifi-standardized-name-probable', v_content ) ) loop
    select reports.f_csv_name( v_node ) into v_name;
    if v_name is not null then
      select array_append( v_names, v_name ) into v_names;
    end if;
  end loop;

  if array_length( v_names, 1 ) > 0 then
    return array_to_string( v_names, '|' );
  end if;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_expiration(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_expiration(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_exp text;
        v_content xml;

/*
<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_status(160182314);
   xml.f_ucid2id('US-5551212-A'));
*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  foreach v_node in array ( select xml.f_findNodes('//ifi-patent-status', v_content) ) loop
    select xml.f_getAttribute('anticipated-expiration', v_node) into v_exp;
    if v_exp is not null then
      return v_exp;
    end if;
  end loop;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_publication_type(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_publication_type(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_type text;
        v_content xml;

/*

<ifi-integrated-content publication-type="G" accession-number="02756518" document-category="Utility" subject-area="Mechanical">
 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_publication_type(xml.f_ucid2id('US-5551212-A'));
 select reports.f_csv_ifi_publication_type( xml.f_ucid2id('US-9165295-B2') );
*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  select xml.f_getAttribute('publication-type', v_content) into v_type;

  if v_type is not null then
    return v_type;
  end if;  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_ifi_status(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ifi_status(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_status text;
        v_content xml;

/*
<ifi-integrated-content>
  <ifi-patent-status anticipated-expiration="20230724">
    <ifi-patent-status-description>Active</ifi-patent-status-description>
  </ifi-patent-status><ifi-parties>
  <ifi-standardized-name country="US" type="Company or Corporation"><addressboo (...)"

 select * from xml.t_ifi_integrated_content limit 10;
 select reports.f_csv_ifi_assignee_current(xml.f_ucid2id('US-5551212-A'));
*/

begin

  select content into v_content from xml.t_ifi_integrated_content where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;

  if( xml.f_nodeExists('//ifi-patent-status-description', v_content) ) then
    return xml.f_textContent('//ifi-patent-status-description', v_content);
  end if;
  
  return reports.f_csv_util_msg_notavail();

end;

$_$;


--
-- Name: f_csv_inpadoc_family(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_inpadoc_family(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

/*
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-0900201-A1') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('US-5120001-A') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('US-5551212-A') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-0698856-A2') );
  family_id = -1
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-2023137-A1') );

*/

declare v_ucid text;
        v_ucids text[] := '{}';

begin

  return (reports.f_csv_util_msg_notavail() );

  for v_ucid in select * from api.f_extended_family( $1 ) order by ucid loop
    select array_append(v_ucids, translate(v_ucid, '-', '')) into v_ucids;
  end loop;

  if (array_length(v_ucids, 1) is null or array_length(v_ucids, 1) < 1 ) then
    return (reports.f_csv_util_msg_notavail() );
  end if;

  return array_to_string( v_ucids, ' | ' );

end;
$_$;


--
-- Name: f_csv_inpadoc_family(integer, boolean); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_inpadoc_family(integer, boolean) RETURNS text
    LANGUAGE plpgsql
    AS $$
declare v_ucid text;
        v_ucids text[] := '{}';
begin
  return (reports.f_csv_util_msg_notavail() );
end;
$$;


--
-- Name: f_csv_inpadoc_family_ex(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_inpadoc_family_ex(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

/*
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-0900201-A1') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('US-5120001-A') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('US-5551212-A') );
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-0698856-A2') );
  family_id = -1
select reports.f_csv_inpadoc_family( xml.f_ucid2id('EP-2023137-A1') );

*/

declare v_ucid text;
        v_ucids text[] := '{}';

begin

  return (reports.f_csv_util_msg_notavail() );

  for v_ucid in select * from api.f_extended_family( $1 ) order by ucid loop
    select array_append(v_ucids, translate(v_ucid, '-', '')) into v_ucids;
  end loop;

  if (array_length(v_ucids, 1) is null or array_length(v_ucids, 1) < 1 ) then
    return (reports.f_csv_util_msg_notavail() );
  end if;

  return array_to_string( v_ucids, ' | ' );

end;
$_$;


--
-- Name: f_csv_inventor_first(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_inventor_first(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121203 -- jmb -- exactly the same as reports.f_csv_inventor() except it only grabs the first name
-- assembles inventor info using the following priority:
-- 		1) "original" first-name and last-name
--		2) "original" just <last-name> ... no first-name available
--		3) "epo" just <name>
--		4) "epo" and just <last-name>
--		5) otherwise returns 'None'

declare v_name record;
        v_names text[];
        
begin

  	for v_name in  
  		select unnest(xpath('//parties/inventors/inventor[@format="original"]/addressbook/last-name/text()', content))::text as lastname, 
 	       unnest(xpath('//parties/inventors/inventor[@format="original"]/addressbook/first-name/text()', content))::text as firstname
        from xml.t_parties
        where publication_id=$1 limit 1 loop
        
    	v_names = array_append(v_names,concat(v_name.lastname, ', ', v_name.firstname));
  	end loop;
 
    	-- grab format=original <last-name>
	if v_names is null
	then
		for v_name in  
  			select unnest(xpath('//inventor[@format="original"]//last-name/text()', content))::text as lastname
        	from xml.t_parties
        	where publication_id=$1 limit 1 loop
        
    		v_names = array_append(v_names,v_name.lastname);
  		end loop;
  	end if;	

	-- grab format=epo <name>
	if v_names is null
	then
		for v_name in  
  			select unnest(xpath('//inventor[@format="epo"]//name/text()', content))::text as lastname
        	from xml.t_parties
        	where publication_id=$1 limit 1 loop
        
    		v_names = array_append(v_names,v_name.lastname);
  		end loop;
  	end if;	
  	
	-- grab format=epo <last-name>
	if v_names is null
	then
		for v_name in  
  			select unnest(xpath('//inventor[@format="epo"]//last-name/text()', content))::text as lastname
        	from xml.t_parties
        	where publication_id=$1 limit 1 loop
        
    		v_names = array_append(v_names,v_name.lastname);
  		end loop;
  	end if;	
 
 	-- if still null ... return "NA message"
	if v_names is null
	then     
	    -- Not Available Msg set in separate function
    	v_names = array_append(v_names, reports.f_csv_util_msg_notavail());
  	end if;	
 
 	
    return array_to_string( v_names, ' | ');
 
end;
$_$;


--
-- Name: f_csv_inventors(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_inventors(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_node xml;
        v_name text;
        v_names text[] := '{}';
        v_has_epo boolean := false;
        v_content xml;

-- select reports.f_csv_inventors(xml.f_ucid2id('US-5551212-A'));

begin

  select content into v_content from xml.t_parties where publication_id=$1;
  if not found then
    return reports.f_csv_util_msg_notavail();
  end if;
  
  foreach v_node in array
    ( select xml.f_findNodes('//inventor[@format="epo"]', v_content) ) loop

    select reports.f_csv_name(v_node) into v_name;
    
    if v_name is not null then
      v_has_epo := true;
      select array_append(v_names, v_name) into v_names;
    end if;
    
  end loop;

  -- if EPO format doesn't exist, settle for original
  if (v_has_epo = false) then
    foreach v_node in array
      ( select xml.f_findNodes('//inventor[@format="original"]', v_content) ) loop
      
      select reports.f_csv_name(v_node) into v_name;
    
      if v_name is not null then
        select array_append(v_names, v_name) into v_names;
      end if;
    end loop;
  end if;
  
  if array_length(v_names, 1) > 0 then
    return array_to_string(v_names, ' | ');
  end if;
  
  return reports.f_csv_util_msg_notavail();
  
end;
$_$;


--
-- Name: f_csv_ipc_all(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ipc_all(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare my_content xml;
	my_array text [];
	my_output text[];
	my_tmp text;
	my_ipc_edition text;
	
-- update 20121203 -- jmb ... added reformatting to padded MPI format
-- hacked 20121207 -- jmb removed reformatting
    --added "not available" msg
-- rewritten 20121209 -- jmb -- read from db once and check for null return
-- select reports.f_csv_ipc_all( xml.f_ucid2id('US-5551212-A') );

begin
    -- read content from db into my_content
    select content into my_content
    from xml.t_classification_ipc
    where publication_id=$1;
    
    -- check to see if I got something
    if found then    
        -- extract ipc edition
        select unnest(xpath('//classification-ipc/edition/text()',my_content))::text into my_ipc_edition;
        if (my_ipc_edition is not null) then
	           my_output := array_append ( my_output, concat('IPCv',my_ipc_edition));
        end if;   
  
        -- extract data from main-classification
        select xpath('//classification-ipc/main-classification/text()',my_content)::text into my_array;      
        if (my_array is not null) then
           foreach my_tmp in array my_array
	       loop
	           my_output := array_append ( my_output, regexp_replace(my_tmp,' +',' ', 'g'));
            end loop;
        end if;
             
        -- extract data from further-classification
        select xpath('//classification-ipc/further-classification/text()', my_content)::text into my_array;       
        if (my_array is not null) then
            foreach my_tmp in array my_array
	        loop
	           my_output := array_append ( my_output, regexp_replace(my_tmp,' +', ' ', 'g'));
            end loop;  
        end if;
        
    end if;

    -- check for null output
    if my_output is null
    then
        return(reports.f_csv_util_msg_notavail());
    else
        return ( array_to_string(my_output, ' | ' ));
    end if;
end;
$_$;


--
-- Name: f_csv_ipc_p(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ipc_p(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

--updated 20121203 -- jmb -- reformatted IPC output
-- hacked 20121206 -- jmb -- removed IPC reformatting per client request
    -- added "not available message.
-- rewritten 20121209 -- jmb -- read from db once and check for null return
-- 
-- DEPRECATED -- 20121212 -- no longer in use  -- jmb

declare my_content xml;
	my_array text [];
	my_output text[];
	my_tmp text;

begin
    -- read content from db into my_content
    select content into my_content
    from xml.t_classification_ipc
    where publication_id=$1;
    
    -- check to see if I got something
    if found
    then   
        -- this is the original main-IPC routine 
        -- extract data from main-classification
        ---select xpath('//classification-ipc/main-classification/text()',my_content)::text into my_array;
        -- grab 1st entry ... 
	    ---my_output := array_append ( my_output, my_array[1]);
	    
	    ---temp routine grabbing all IPC
	           -- extract data from further-classification
        select xpath('//classification-ipc/further-classification/text()', my_content)::text into my_array;
        foreach my_tmp in array my_array
	    loop
	       my_output := array_append ( my_output, my_tmp);
        end loop;   
  
    end if;

    -- check for null output
    if my_output is null
    then
        return(reports.f_csv_util_msg_notavail());
    else
        return ( array_to_string(my_output, '|' ));
    end if;
end;
$_$;


--
-- Name: f_csv_ipc_pad(text); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ipc_pad(my_tmp text) RETURNS text
    LANGUAGE plpgsql
    AS $$
-- hack to reformat IPC and ECLA codes -- 20121203 -- jmb

-- DEPRECATED 20121212 jmb

declare ipc_raw	text;
	ipc_base text;
	ipc_middle text;
	ipc_end text;
	ipc_out text;

begin

    ipc_raw = my_tmp;    
    ipc_base = substr(ipc_raw,0,strpos(ipc_raw,' '));
    ipc_middle = lpad(ltrim(substr(ipc_raw,strpos(ipc_raw,' '),strpos(ipc_raw,'/')-strpos(ipc_raw,' ')),' '),4,'0');
    ipc_end = substr(ipc_raw,strpos(ipc_raw,'/')+1);

	ipc_out = concat(ipc_base,ipc_middle,ipc_end);
    
    return  ipc_out;
end;
$$;


--
-- Name: f_csv_ipcr_all(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_ipcr_all(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare my_content xml;
	my_array text [];
	my_output text[];
	my_tmp text;
	
-- 20121212 jmb -- first cut
-- IPCR function !!!!


begin
    -- read content from db into my_content
    select content into my_content
    from xml.t_classifications_ipcr
    where publication_id=$1;
    
    -- check to see if I got something
    if found then    
          
        select xpath('//classification-ipcr/text()', my_content)::text into my_array;
        
        if (my_array is null) then
          return ( reports.f_csv_util_msg_notavail() );
        end if;
        
        foreach my_tmp in array my_array
	    loop
	       my_output := array_append ( my_output, regexp_replace(rtrim(regexp_replace(rtrim(my_tmp),'\d{8}.*$', '' )),' +',' ','g'));
        end loop;        
  
    end if;

    -- check for null output
    if my_output is null
    then
        return(reports.f_csv_util_msg_notavail());
    else
        return ( array_to_string(my_output, ' | ' ));
    end if;
end;
$_$;


--
-- Name: f_csv_legal_status(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_legal_status(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121206 jmb

declare my_events record;
	my_tmp text;
	my_output text[];
	my_int int := 1;
	my_xpath text;
	my_content xml;
	my_event_text text;
	my_event_attributes text [];

begin

    select content from xml.t_legal_status into my_content where publication_id=$1;
    
    for my_events in 
    
        select unnest(xpath('//legal-event/@country', my_content))::text as country,
            unnest(xpath('//legal-event/@code', my_content))::text as code,
            unnest(xpath('//legal-event/@date', my_content))::text as date
            loop
    
    
               my_xpath := concat('//legal-event[', my_int, ']/legal-event-body/event-title/text()');
               select unnest(xpath(my_xpath,my_content))::text into my_tmp;
               my_event_text := concat(my_event_text, my_tmp);
         
               -- ONLY take the first event-attribute and all of its label/value pairs
               my_xpath = concat('//legal-event[', my_int, ']/legal-event-body/event-attributes/event-attribute/*/text()');
                select xpath(my_xpath,my_content)::text into my_event_attributes;
                foreach my_tmp in array(my_event_attributes) loop
                    if my_tmp not like 'free format text%'
                    then
                        my_event_text = concat(my_event_text, ' - ', my_tmp);
                    end if;
                end loop;
    
                --construct output record
                my_output = array_append(my_output, concat('Date: ', 
                    my_events.date, 
                    ' Country: ',
                    my_events.country,
                    ' Code: ',
                    my_events.code,
                    ' Desc: ',
                    ' ',
                    my_event_text
                    ));    
                    
                    my_tmp = '';
                    my_event_text := null;
                    my_int = my_int +1;

            end loop;        


  if my_output is null
  then
    return(reports.f_csv_util_msg_notavail());
  else
    return ( array_to_string(my_output, ' | ' ));
  end if;
end;
$_$;


--
-- Name: f_csv_load_id(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_load_id(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_modified_load_id int;

begin

-- select reports.f_csv_load_id( xml.f_ucid2id('EP-0700000-B1') );

    select modified_load_id into v_modified_load_id
    from xml.t_patent_document_values
    where publication_id=$1;

    return v_modified_load_id;
    
end;
$_$;


--
-- Name: f_csv_name(xml); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_name(xml) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- INPUT: xml subtree of either assignee, applicant, inventor or agent
-- Return text of either /addressbook/name or
-- either orgname or a concatenation of last-name, first-name

declare v_node alias for $1;

/*
 TEST
 --------
 select
   reports.f_csv_name(
     ( xml.f_getElementsByTagName('agent',
         ( select content from xml.t_parties where publication_id=xml.f_ucid2id('CN-101642422-B'))
       )
     )[1]
  );
*/
begin

  return (
    select
      case
        when xml.f_nodeExists('//addressbook/name', v_node)
          then xml.f_textContent('//addressbook/name', v_node)
        when xml.f_nodeExists('//addressbook/orgname', v_node)
          then xml.f_textContent('//addressbook/orgname', v_node)
        when ( xml.f_nodeExists('//addressbook/last-name', v_node) and xml.f_nodeExists('//addressbook/first-name', v_node) )
          then xml.f_textContent('//addressbook/last-name', v_node) || ', ' || xml.f_textContent('//addressbook/first-name', v_node)
        else xml.f_textContent('//addressbook/last-name', v_node)
      end )::text;
end;
$_$;


--
-- Name: f_csv_non_patent_citations(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_non_patent_citations(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- added 2012/12/05 00 jmb ... 

-- sometimes doesn't return null instead of "not available" message?? e.g pubid = 119030546 works and pubid= 119009001 doesn't
-- DS: 119009001 has content 'None' so it does work.

declare my_ucid text;
     my_input text [];
     my_output text [];
     
-- select reports.f_csv_non_patent_citations(119030546);
-- select reports.f_csv_non_patent_citations(119009001);
begin

	select xpath('//nplcit/text/text()', content)::text
	  into my_input
	from xml.t_citations
	where publication_id=$1;
	
	if my_input is not Null
	then	
	  foreach my_ucid in array my_input loop
	     select array_append(my_output, my_ucid) into my_output;
	  end loop; 
	  
	end if;
	
	if my_output is not null
	then
	  return array_to_string( my_output, ' | ');
	else  
	  return(reports.f_csv_util_msg_notavail());
	end if;
	
end;
$_$;


--
-- Name: f_csv_patent_citations(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_patent_citations(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_node xml;
        v_ucid text;
        v_output text [];
     

begin

  -- select reports.f_csv_patent_citations( xml.f_ucid2id('US-5551212-A') );
  select content into v_content from xml.t_citations where publication_id=$1;
  if not found then
    return(reports.f_csv_util_msg_notavail());
  end if;

  foreach v_node in array
    ( select xml.f_findNodes('//patcit', v_content ) ) loop

    select xml.f_getAttribute('ucid', v_node) into v_ucid;
    if v_ucid is not null then
      select array_append(v_output, v_ucid) into v_output;
    end if;
  end loop;

  if    (v_output is null)
     or (array_length(v_output, 1) is null)
     or (array_length(v_output, 1) < 1)	then
    return(reports.f_csv_util_msg_notavail());
  end if;
  
  return array_to_string( v_output, ' | ');
	
end;
$_$;


--
-- Name: f_csv_pct_application_number(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_pct_application_number(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121205 --jmb ... extricate PCT app out of priority data if available

declare v_ucid text;
        v_att text;
        v_atts text[];
        v_ucids text[];

begin
  select xpath('//priority-claims/priority-claim/@ucid', content)::text
    into v_atts
  from xml.t_priority_claims
  where publication_id=$1;

  if (v_atts is null) then
    return ( reports.f_csv_util_msg_notavail());
  end if;
  
  foreach v_att in array v_atts loop
    if ( substr( v_att, 0, 2) = 'WO' )
    then
        select array_append(v_ucids, v_att) into v_ucids;
    end if;
  end loop;
  
   	-- if no data return NA msg otherwise return data  
    if v_ucids is null
    then
        -- Not Available Msg set in separate function
  		return reports.f_csv_util_msg_notavail();
  	else
  		   return replace(array_to_string(v_ucids , ' | '),' ','.');
    end if;
  --return array_to_string( v_ucids, '|' );
  
end;
$_$;


--
-- Name: f_csv_priority_country(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_priority_country(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_node xml;
        v_ucid text;
        v_countries text[];

begin

  select content into v_content from xml.t_priority_claims where publication_id=$1;
  if v_content is null then
    return ( reports.f_csv_util_msg_notavail());
  end if;
  
  foreach v_node in array
    ( select xml.f_findNodes('//priority-claims/priority-claim', v_content) ) loop

    -- if there is no ucid, don't cature
    select xml.f_getAttribute( 'ucid', v_node ) into v_ucid;
    if v_ucid is not null then
      select array_append( v_countries, substring(v_ucid from 1 for 2) ) into v_countries;
    end if;
  end loop;

  return array_to_string( v_countries, ' | ' );
end;
$_$;


--
-- Name: f_csv_priority_date(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_priority_date(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_txt_dates text[];
        v_content xml;
        v_node xml;
        v_fmt text;

-- select reports.f_csv_priority_date ( xml.f_ucid2id('EP-1797537-B1') );
-- select reports.f_csv_priority_date ( xml.f_ucid2id('US-20100212873-A1') );

begin
  select content into v_content from xml.t_priority_claims where publication_id=$1;
  if not found then
    return ( reports.f_csv_util_msg_notavail());
  end if;

  foreach v_node in array
      ( select xml.f_findNodes( '//priority-claims/priority-claim/document-id', v_content ) ) loop

    -- @format must be epo as non-docdb priorities are filtered
    select xml.f_getAttribute( 'format', v_node ) into v_fmt;
    if v_fmt = 'epo' then
      select array_append( v_txt_dates, to_char( xml.f_textContent('date', v_node )::date, 'yyyy-mm-dd' ) ) into v_txt_dates;
    end if;
  end loop;
  return array_to_string( v_txt_dates, ' | ' );
end;

$_$;


--
-- Name: f_csv_priority_number(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_priority_number(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_ucid text;
        v_att text;
        v_atts text[] = '{}';
        v_ucids text[];

begin
  select xpath('//priority-claims/priority-claim/@ucid', content)::text
    into v_atts
  from xml.t_priority_claims
  where publication_id=$1;

  if (v_atts is null or array_length(v_atts, 1) is null) then
    return ( reports.f_csv_util_msg_notavail());
  end if;
  
  foreach v_att in array v_atts loop
    select array_append(v_ucids, v_att) into v_ucids;
  end loop;
  
  return array_to_string( v_ucids, ' | ' );
end;
$_$;


--
-- Name: f_csv_priority_years(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_priority_years(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_content xml;
        v_node xml;
        v_dates text[];
        v_years text[];
        v_date text;
        v_year text;
        v_fmt text;

-- select reports.f_csv_priority_years ( xml.f_ucid2id('EP-1797537-B1') );
-- select reports.f_csv_priority_years ( xml.f_ucid2id('US-20100212873-A1') );

begin

  select content into v_content from xml.t_priority_claims where publication_id=$1;
  if not found then
    return ( reports.f_csv_util_msg_notavail());
  end if; 

  foreach v_node in array
      ( select xml.f_findNodes( '//priority-claims/priority-claim/document-id', v_content ) ) loop
    
    -- @format must be epo as non-docdb priorities are filtered
    select xml.f_getAttribute( 'format', v_node ) into v_fmt; 
    if v_fmt = 'epo' then
      select xml.f_textContent('./date', v_node) into v_date;
      select date_part('year', v_date::date) into v_year;
      select array_append(v_years, v_year) into v_years;
    end if;
  end loop;
    
  return array_to_string( v_years, ' | ' );
end;
$_$;


--
-- Name: f_csv_publication_country(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_country(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_ctry text;

begin
  select country into v_ctry
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return v_ctry;
end;
$_$;


--
-- Name: f_csv_publication_date(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_date(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_date date;

begin
-- select reports.f_csv_publication_date( xml.f_ucid2id( 'US-5551212-A') );
  select published into v_date
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return to_char( v_date, 'YYYY-MM-DD')::text;
end;
$_$;


--
-- Name: f_csv_publication_kind(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_kind(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_kind text;

begin
-- select * from xml.t_patent_document_values limit 1;
  select kind into v_kind
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return v_kind;
end;
$_$;


--
-- Name: f_csv_publication_number(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_number(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_dnum text;

-- select reports.f_csv_publication_number( xml.f_ucid2id( 'US-5551212-A' ) );
begin
  select doc_number into v_dnum
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return v_dnum;
end;
$_$;


--
-- Name: f_csv_publication_ucid(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_ucid(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_ucid text;

-- select reports.f_csv_publication_ucid( xml.f_ucid2id( 'US-5551212-A' ) );
begin
  select ucid into v_ucid
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return v_ucid;
end;
$_$;


--
-- Name: f_csv_publication_year(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_publication_year(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_year integer;

begin
-- select * from xml.t_patent_document_values limit 1;
  select date_part('year', published)::integer into v_year
  from xml.t_patent_document_values
  where publication_id=$1 limit 1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;
  return v_year;
end;
$_$;


--
-- Name: f_csv_related_applications(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_related_applications(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

/* The content model:

<!ELEMENT relation (document-id | (child-doc, parent-doc)) >
<!ATTLIST relation
  type (a-371-of-international | addition | division | division-into | continuation | continuation-in-part |
        continuing-reissue | reissue | us-divisional-reissue | reexamination |
        us-reexamination-reissue-merger | substitution | us-provisional-application |
        utility-model-basis | correction | related-publication) #REQUIRED
>

<!ELEMENT parent-doc (document-id+, parent-grant-document?, parent-pct-document?) >
<!ATTLIST parent-doc
  ucid CDATA #REQUIRED
  psta (granted|pending|abandoned|SIR|unknown) 'unknown'
>
<!ELEMENT parent-grant-document (document-id+) >
<!ATTLIST parent-grant-doc
  ucid CDATA #REQUIRED
>
<!ELEMENT parent-pct-document (document-id+) >
<!ATTLIST parent-pct-doc
  ucid CDATA #REQUIRED
>
<!ELEMENT child-doc (document-id+) >
<!ATTLIST child-doc
  ucid CDATA #REQUIRED
>
*/

declare v_record record;
        v_content xml;
        v_relation xml;
        v_attr text;
        v_relation_array text[] := '{}';
        v_rel_array text[] := '{}';
        v_relation_text text := null;
        v_elements text[];
        v_element text;
        v_path text;

-- CHILD (DATE) PARENT (DATE) PARENT-PUB (DATE) TYPE 
-- select reports.f_csv_related_applications( xml.f_ucid2id('US-5551212-A') );

begin

  select content into v_content from xml.t_related_documents where publication_id=$1;
  if not found then
    return ( reports.f_csv_util_msg_notavail() );
  end if;

  -- materialize our record
  select ''::text as related_pub_ucid, ''::text as related_pub_date,
         ''::text as child_ucid, ''::text as child_date,
         ''::text as parent_ucid, ''::text as parent_date,
         ''::text as parent_pub_ucid, ''::text as parent_pub_date,
         ''::text as parent_pct_ucid, ''::text as parent_pct_date into v_record;

  foreach v_relation in array
    ( select xml.f_findNodes('//relation', v_content) ) loop
    
    -- the relationship
    v_attr := xml.f_getAttribute('type', v_relation);

      -- This is type related-publication
    if ( xml.f_nodeExists('./document-id', v_relation) ) then
      select coalesce(xml.f_textContent('./document-id/country', v_relation), '') || '-' ||
             coalesce(xml.f_textContent('./document-id/doc-number', v_relation), '') || '-' ||
             coalesce(xml.f_textContent('./document-id/kind', v_relation), '') into v_record.child_ucid;
      select xml.f_textContent('./document-id/date', v_relation) into v_record.related_pub_date;
    end if;
    if ( xml.f_nodeExists('//child-doc', v_relation) ) then
      select xml.f_textContent('//child-doc/@ucid', v_relation) into v_record.child_ucid;
      select xml.f_textContent('//child-doc/document-id/date', v_relation) into v_record.child_date;
    end if;
    if ( xml.f_nodeExists('//parent-doc', v_relation) ) then
      select xml.f_textContent('//parent-doc/@ucid', v_relation) into v_record.parent_ucid;
      select xml.f_textContent('//parent-doc/document-id/date', v_relation) into v_record.parent_date;

      if ( xml.f_nodeExists('//parent-doc/parent-grant-document', v_relation) ) then
        select xml.f_textContent('//parent-doc/parent-grant-document/@ucid', v_relation) into v_record.parent_pub_ucid;
        select xml.f_textContent('//parent-doc/parent-grant-document/document-id/date', v_relation) into v_record.parent_pub_date;
      end if;
      if ( xml.f_nodeExists('//parent-doc/parent-pct-document', v_relation) ) then
        select xml.f_textContent('//parent-doc/@ucid', v_relation) into v_record.parent_pct_ucid;
        select xml.f_textContent('//parent-doc/document-id/date', v_relation) into v_record.parent_pct_date;
      end if;
    end if;

    -- push the values
    select array_append(v_rel_array,
               coalesce(v_record.child_ucid, '')      || ' ' || coalesce(v_record.child_date, '')      || ' ' ||
               coalesce(v_record.parent_ucid, '')     || ' ' || coalesce(v_record.parent_date, '')     || ' ' ||
               coalesce(v_record.parent_pub_ucid, '') || ' ' || coalesce(v_record.parent_pub_date, '') || ' ' ||
               coalesce(v_record.parent_pct_ucid, '') || ' ' || coalesce(v_record.parent_pct_date, '') || ' ' || v_attr ) into v_rel_array;
  end loop;
  

  if ( (v_rel_array is null) or (array_length(v_rel_array, 1) is null) or (array_length(v_rel_array, 1) < 1) ) then
    return ( reports.f_csv_util_msg_notavail() );
  else
    return ( array_to_string(v_rel_array, ' | ') );
  end if;

end;
$_$;


--
-- Name: f_csv_title_de(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_title_de(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- DEPRECATED 20121212 jmb

begin
  return (
    select unnest(xpath('//invention-title[@lang="DE"]/text()', content))::text
    from xml.t_invention_title
    where publication_id=$1
    limit 1
 );
end;
$_$;


--
-- Name: f_csv_title_en(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_title_en(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

declare v_rec record;
        v_cnt integer;
        v_lang text;
        v_ord integer;
        v_content xml;

-- select reports.f_csv_title_en( xml.f_ucid2id('US-5551212-A') );
--  *** Dev has no title for the EP document below ...
-- select reports.f_csv_title_en( xml.f_ucid2id('EP-0700001-B1') );
-- select reports.f_csv_title_en( xml.f_ucid2id('EP-1537304-A1') );
/*
  select * from xml.t_invention_title where publication_id=xml.f_ucid2id('EP-0700001-B1');
  select * from xml.t_patent_document_values where publication_id=xml.f_ucid2id('EP-0700001-B1');
  update xml.t_patent_document_values set lang='FR' where publication_id=xml.f_ucid2id('EP-0700001-B1');
  delete from xml.t_invention_title where invention_title_id=112399;
*/
begin

  create temp table ttl_tbl ( pub_lang varchar(2), ttl_lang varchar(2), x xml) on commit drop;

  for v_rec in
        select t1.lang as pub_lang, t2.*
        from xml.t_patent_document_values t1
          inner join xml.t_invention_title t2
            on (t1.publication_id=t2.publication_id)
        where t1.publication_id=$1 loop
    insert into ttl_tbl (pub_lang, ttl_lang, x)
      values ( v_rec.pub_lang,
               xml.f_getAttribute('lang', v_rec.content),
               v_rec.content
      );
  end loop;

  select ttl_lang,
    case when ttl_lang = 'EN' then 0
         when ttl_lang = pub_lang then 1
         when ttl_lang = 'DE' then 2
         when ttl_lang = 'FR' then 3
         else 4
    end as ord, x
    into v_lang, v_ord, v_content
  from ttl_tbl
  order by ord limit 1;

  select count(*) into v_cnt from ttl_tbl;

  if (v_cnt > 0) then
    if (xml.f_nodeExists('//invention-title[@lang="'||v_lang||'"]', v_content) ) then
      return ( xml.f_findValue('//invention-title[@lang="'||v_lang||'"]', v_content) );
    end if;
  end if;
  
  /*
  for v_content in select content from xml.t_invention_title where publication_id=$1 loop
    if (xml.f_nodeExists('//invention-title[@lang="EN"]', v_content) ) then
      return ( xml.f_findValue('//invention-title[@lang="EN"]', v_content) );
    end if;
  end loop;

  -- if we are here ... there is no english title
  -- Grab language
  select lang into v_lang from xml.t_patent_document_values where publication_id=$1;
  for v_content in select content from xml.t_invention_title where publication_id=$1 loop
    if (xml.f_nodeExists('//invention-title[@lang="'||v_lang||'"]', v_content) ) then
      return ( xml.f_findValue('//invention-title[@lang="'||v_lang||'"]', v_content) );
    end if;
  end loop;
  */
  return ( reports.f_csv_util_msg_notavail() );
end;
$_$;


--
-- Name: f_csv_title_es(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_title_es(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- DEPRECATED 20121212 jmb

begin
  return (
    select unnest(xpath('//invention-title[@lang="ES"]/text()', content))::text
    from xml.t_invention_title
    where publication_id=$1
    limit 1
 );
end;
$_$;


--
-- Name: f_csv_title_fr(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_title_fr(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$

-- DEPRECATED 20121212 jmb

begin
  return (
    select unnest(xpath('//invention-title[@lang="FR"]/text()', content))::text
    from xml.t_invention_title
    where publication_id=$1
    limit 1
 );
end;
$_$;


--
-- Name: f_csv_us_class_all(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_us_class_all(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked 20121203 -- jmb -- added support for "Not Available"
-- hacked 20121207 -- jmb -- removed mpi padding
declare my_var text;
	my_array text[];

begin
    for my_var in select unnest(xpath('//classification-national/main-classification/text()', content))::text
    	from xml.t_classification_national
    	where publication_id=$1 
    loop
    	my_array = array_append(my_array, my_var);
    end loop;
    
    for my_var in select unnest(xpath('//classification-national/further-classification/text()', content))::text
       	from xml.t_classification_national
    	where publication_id=$1 
    loop
    		my_array = array_append(my_array, my_var);

    end loop;
  
  	-- if no data return NA msg otherwise return data  
    if my_array is null
    then
        -- Not Available Msg set in separate function
  		return reports.f_csv_util_msg_notavail();
  	else
  		   return array_to_string(my_array, ' | ');
    end if;
    
 
end;
$_$;


--
-- Name: f_csv_us_class_p(integer); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_us_class_p(integer) RETURNS text
    LANGUAGE plpgsql
    AS $_$
-- hacked - 20121203 -- jmb
-- return "Not Available" if no value
-- hacked 20121207 jmb -- removed padding

declare my_var text;
	my_array text[];

begin
    for my_var in select unnest(xpath('//classification-national/main-classification/text()', content))::text
    	from xml.t_classification_national
    	where publication_id=$1 
    		limit 1
    loop
    	my_array = array_append(my_array, my_var);
    end loop;
 
   	-- if no data return NA msg otherwise return data  
    if my_array is null
    then
        -- Not Available Msg set in separate function
  		return reports.f_csv_util_msg_notavail();
  	else
  		   return array_to_string(my_array, ' | ');
    end if;

end;
$_$;


--
-- Name: f_csv_util_msg_notavail(); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_csv_util_msg_notavail() RETURNS text
    LANGUAGE plpgsql
    AS $$
-- common message to insert in cell when data not available -- 20121203 -- jmb
-- 20121212 -- jmb -- changed 'Not Available' to 'No Data'

begin
  
    return  'No Data';

end;
$$;


--
-- Name: f_on_validation_report_update(); Type: FUNCTION; Schema: reports; Owner: -
--

CREATE FUNCTION reports.f_on_validation_report_update() RETURNS trigger
    LANGUAGE plpgsql
    AS $$

declare 
        -- these are for filling in NEW row on match or auto-complete
        v_publication_id integer;
        v_ucid varchar(32);
        v_published date;
        v_title text;
        v_ord integer;
        v_withdraw boolean;
        v_dup boolean;
        -- next, if no match
        v_country char(2);
        v_doc_number varchar(32);
        v_count integer;
        v_rec record;
        v_rec_titles record;
        v_suggestions reporting.suggestions_t[];
        v_ref_ucid varchar(64);
        
begin

-- truncate table web.t_60daa494899dbeebb78e1d79dad349c3af515053
-- insert into  web.t_60daa494899dbeebb78e1d79dad349c3af515053 (user_input, formatted_input) values ('EP0700000A2', 'EP-0700000-A2');
-- insert into  web.t_60daa494899dbeebb78e1d79dad349c3af515053 (user_input, formatted_input) values ('EP0700000B', 'EP-0700000-B');
-- insert into  web.t_60daa494899dbeebb78e1d79dad349c3af515053 (user_input, formatted_input) values ('AR240768A', 'AR-240768-A');
-- update web.t_60daa494899dbeebb78e1d79dad349c3af515053 set formatted_input='AR-240768-A2' where rowid=144;
-- select * from web.t_60daa494899dbeebb78e1d79dad349c3af515053;

  if (TG_OP = 'INSERT') then
    -- if formatted_input is null, this is an ADD_ROW() call from UI
    if (NEW.formatted_input is null) then
      NEW.matched := false; -- default
      NEW.publication_id := null;
      NEW.formatted_input = 'xx-xxxxxxxx-xx';
      NEW.published := null;
      NEW.title := null;
      NEW.suggestions := null;
      return NEW;
    end if;
  end if;

  -- easiest case is a MATCH ... Note the left join because
  -- although we display a title, we are really after publication_id
  select t1.publication_id, t1.ucid, t1.published, t1.withdraw,
         (xpath('/invention-title/text()', t2.content))[1] as title,
         case when (xpath('/invention-title/@lang', t2.content))[1]::text = 'EN' then 0
              when (xpath('/invention-title/@lang', t2.content))[1]::text = 'FR' then 1
              when (xpath('/invention-title/@lang', t2.content))[1]::text = 'DE' then 2
              else 3
         end as langord
     into v_publication_id,v_ucid,v_published,v_withdraw,v_title,v_ord
  from xml.t_patent_document_values t1
    left outer join xml.t_invention_title t2 on (t1.publication_id=t2.publication_id)
  where t1.publication_id=xml.f_ucid2id(NEW.formatted_input)
  order by langord asc
  limit 1; -- pick the best language ...
  
  if found then
    NEW.matched = true;
    NEW.publication_id := v_publication_id;
    NEW.formatted_input := v_ucid;
    NEW.published := v_published;
    NEW.title := v_title;
    NEW.suggestions := null;
    if v_withdraw = true then
      NEW.title := 'WITHDRAWN';
      NEW.status := 'w'; -- withdrawn
    end if;
    -- raise warning 'return %: %', TG_OP, NEW;
    return NEW;
  end if;

  -- We are here because the formatted_input was not found. Get components ...
  select split_part(NEW.formatted_input, '-', 1) into v_country;
  select split_part(NEW.formatted_input, '-', 2) into v_doc_number;

  -- First: if there is only one publication per country+doc_number, use it as an auto-complete feature
  select count(*) into v_count from xml.t_patent_document_values where country=v_country and doc_number=v_doc_number;
  if v_count = 1 then
    select t1.publication_id, t1.ucid, t1.published, t1.withdraw,
           (xpath('/invention-title/text()', t2.content))[1] as title,
           case when (xpath('/invention-title/@lang', t2.content))[1]::text = 'EN' then 0
                when (xpath('/invention-title/@lang', t2.content))[1]::text = 'FR' then 1
                when (xpath('/invention-title/@lang', t2.content))[1]::text = 'DE' then 2
                else 3
           end as langord
       into v_publication_id,v_ucid,v_published,v_withdraw,v_title,v_ord
    from xml.t_patent_document_values t1
      left outer join xml.t_invention_title t2 on (t1.publication_id=t2.publication_id)  
    where t1.country=v_country and t1.doc_number=v_doc_number
    order by langord asc
    limit 1; -- pick the best language ...

    if found then
      NEW.matched := true;
      NEW.publication_id := v_publication_id;
      NEW.formatted_input := v_ucid;
      NEW.published := v_published;
      NEW.title := v_title;
      NEW.suggestions := null;
      if v_withdraw = true then
        NEW.title := 'WITHDRAWN';
        NEW.status := 'w'; -- withdrawn
      end if;
      -- raise warning 'return %: %', TG_OP, NEW;
      return NEW;
    end if;
  else

    -- either v_count = 0 or v_count > 1
    NEW.matched := false;
    NEW.title := null;
    NEW.published := null;
    NEW.suggestions := null;
  
   -- grab suggestions ...
    for v_rec in
      select t1.publication_id, t1.ucid, t1.published
      from xml.t_patent_document_values t1
      where t1.country=v_country and t1.doc_number=v_doc_number loop

      select (xpath('/invention-title/text()', content))[1] as title,
        case when (xpath('/invention-title/@lang', content))[1]::text = 'EN' then 0
             when (xpath('/invention-title/@lang', content))[1]::text = 'FR' then 1
             when (xpath('/invention-title/@lang', content))[1]::text = 'DE' then 2
             else 3
        end as langord into v_title, v_ord
      from xml.t_invention_title
      where publication_id=v_rec.publication_id
      order by langord asc
      limit 1;

      select array_append( v_suggestions,
        (v_rec.publication_id, v_rec.ucid, v_rec.published, v_title)::reporting.suggestions_t)
          into v_suggestions;
    end loop;

/*
    foreach v_rec in array v_suggestions loop  
    end loop;
*/

    if array_length(v_suggestions, 1) > 0 then
      NEW.suggestions := v_suggestions;
      return NEW;
    end if;

  end if;
  -- raise warning 'return %: %', TG_OP, NEW;
  return NEW;
end;
$$;


SET default_with_oids = false;

--
-- Name: t_report_validations; Type: TABLE; Schema: reports; Owner: -
--

CREATE TABLE reports.t_report_validations (
    input_id integer NOT NULL,
    user_id integer NOT NULL,
    publication_id integer,
    user_input text DEFAULT 'xxxxxxxx'::text,
    matched boolean DEFAULT false,
    formatted_input text,
    duplicate boolean DEFAULT false,
    title text,
    published date,
    format text,
    status text,
    input_type text DEFAULT 'dn'::text,
    suggestions reporting.suggestions_t[]
);


--
-- Name: t_report_validations_input_id_seq; Type: SEQUENCE; Schema: reports; Owner: -
--

CREATE SEQUENCE reports.t_report_validations_input_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: t_report_validations_input_id_seq; Type: SEQUENCE OWNED BY; Schema: reports; Owner: -
--

ALTER SEQUENCE reports.t_report_validations_input_id_seq OWNED BY reports.t_report_validations.input_id;


--
-- Name: t_report_validations input_id; Type: DEFAULT; Schema: reports; Owner: -
--

ALTER TABLE ONLY reports.t_report_validations ALTER COLUMN input_id SET DEFAULT nextval('reports.t_report_validations_input_id_seq'::regclass);


--
-- Name: t_report_validations t_report_validations_pkey; Type: CONSTRAINT; Schema: reports; Owner: -
--

ALTER TABLE ONLY reports.t_report_validations
    ADD CONSTRAINT t_report_validations_pkey PRIMARY KEY (input_id);


--
-- Name: t_report_validations tr_on_validation_report_insert; Type: TRIGGER; Schema: reports; Owner: -
--

CREATE TRIGGER tr_on_validation_report_insert BEFORE INSERT OR UPDATE ON reports.t_report_validations FOR EACH ROW EXECUTE PROCEDURE reports.f_on_validation_report_update();


--
-- PostgreSQL database dump complete
--

